import path from 'node:path';
import multer from 'multer';
import { Environment, Request, File, Request as RequestUtil } from '../util/index.js';
import { AssistantControl } from '../service/index.js';
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        try {
            const basePath = process.env.ASSIST_CONTROL_DB_PATH;
            const { version } = Request.getAllParams(req);
            if (!basePath)
                throw new Error('Invalid base path database');
            if (!version)
                throw new Error('Invalid db file version');
            const dbPath = path.join(basePath, version);
            const successResolver = () => cb(null, dbPath);
            const createDbPath = async () => {
                try {
                    await File.createDir(dbPath);
                    successResolver();
                }
                catch (error) {
                    cb(new Error(`Fail to create path ${dbPath}`));
                }
            };
            File.isDir(dbPath).then(successResolver).catch(createDbPath);
        }
        catch (error) {
            cb(error);
        }
    },
    filename: (req, file, cb) => {
        const { name } = Request.getAllParams(req);
        cb(null, name);
    },
});
const upload = multer({ storage });
class AssistantControlRoutes {
    register(app) {
        app.get('/database', async (req, res) => {
            try {
                const data = await AssistantControl.getDataBaseList();
                res.status(200).send(data);
            }
            catch (err) {
                RequestUtil.handleResponseError(res, err);
            }
        });
        app.get('/database/download', async (req, res) => {
            const params = RequestUtil.getAllParams(req);
            AssistantControl.downloadDb(res, params);
        });
        app.post('/database/upload', upload.any(), async (req, res) => {
            const { version, name } = Request.getAllParams(req);
            const basePath = process.env.ASSIST_CONTROL_DB_PATH || '';
            res.send({
                name,
                version,
                path: path.join(basePath, version),
            });
        });
        app.delete('/database', async (req, res) => {
            try {
                const { name, version } = Request.getAllParams(req);
                const basePath = process.env.ASSIST_CONTROL_DB_PATH;
                if (basePath && version) {
                    const dbPath = path.join(basePath, version);
                    await new File(name, dbPath).remove();
                    res.send({
                        name,
                        version,
                        path: dbPath,
                    });
                }
                else {
                    throw new Error('FAIL_RESOLVE_DB_PATH');
                }
            }
            catch (err) {
                RequestUtil.handleResponseError(res, err);
            }
        });
        app.get('/environment', async (req, res) => {
            try {
                const env = await Environment.load();
                delete env.SECRET_KEY;
                res.send({ env });
            }
            catch (error) {
                console.error('Invalid Environment', process.env.LOADED_ENV);
                RequestUtil.handleResponseError(res, error);
            }
        });
        app.put('/environment', async (req, res) => {
            try {
                const params = RequestUtil.getAllParams(req);
                if (params.text) {
                    await Environment.write(params.text);
                    await Environment.load({ force: true });
                    return res.status(200).send({
                        message: 'Configuração atualizada com sucesso',
                    });
                }
                res.status(400).send({
                    message: 'Parâmetro {text} não informado.',
                });
            }
            catch (error) {
                RequestUtil.handleResponseError(res, error);
            }
        });
        app.get('/updater-packages-sync', async (req, res) => {
            try {
                const loadedPackages = await AssistantControl.loadUpdaterPackages();
                res.send(loadedPackages);
            }
            catch (error) {
                RequestUtil.handleResponseError(res, error);
            }
        });
    }
}
const instance = new AssistantControlRoutes();
export { instance as AssistantControlRoutes };
export default instance;
